#!/usr/bin/perl

#
#	TCP client to read a file from a TCP connection.
#
#	Written to act as a client for an Arduino reading data from
#	a PDP-11 DR11-C, but could be used for most anything. 
#
#

use warnings;
use strict;

use IO::Socket;
use IO::Select;

my $serverPort = 10001;
my $serverAddr = "192.168.42.254";

my $debug = 0;
my $outfile = '';
my $self = $ARGV[0];
our($opt_d,$opt_o);

my $BUFSIZE = 512;
my $buf;
my $len;

my $blkcnt = 0;
my $checksum = 0;

use Getopt::Std;
use FileHandle;

sub usage();

#
#	Process command line arguments
#

getopts('do:') || usage();
if(!$opt_o) {
	usage();
}

$outfile = $opt_o;
$debug = $opt_d;

#
#	Open up the output file
#

if(!open(OFILE,">$outfile")) {
	print "Unable to open $outfile for output.\n";
	flush();
	exit(-2);
}

binmode OFILE;

my $temp_fh = select(OFILE);
$| = 1;
select($temp_fh);

#
#	Connect to the server (typically, an Arduino)
#

my $socket = new IO::Socket::INET (
	PeerHost => $serverAddr,
	PeerPort => $serverPort,
	Proto => 'tcp'
);

if(!$socket) {
	die "Unable to connect to server at $serverAddr on port $serverPort.\n";
}

if($debug) {
	print("Connected to server at $serverAddr on port $serverPort.\n");
}

#	For inexplicable reasons, the Arduino server REQUIRES some data
#	to properly handle a connection from a client.  8P
#
#	So, a quote from Ratchet & Clank's Mr. Zurkon...

$socket -> send("Stinky aliens....");

#
#	Now go into a loop receiving and writing data...
#

while(1) {
	
	#	Get a block
	
	$socket -> recv($buf,$BUFSIZE);
	$len = length($buf);
	if($debug) {
		print("Received Block: $blkcnt, Length: $len\n");
	}
	
	if($len <= 0) {
		last;
	}
	
	#	Write out the block
	
	if(!print OFILE $buf) {
		print "File output error.\n";
	}
	
	#	Add the characters into the checksum
	
	foreach my $c (split //,$buf) {
		$checksum = (($checksum >> 1) & 0x7fff) + (($checksum & 1) << 15) +
			ord($c);
	}
	++$blkcnt;
}

print("Done. $blkcnt blocks received.  Checksum: $checksum\n");
close(OFILE);

#
#	Routine to tell user how to use program.
#
sub usage() {
	print "Usage: perl $self [-d] -o <output file>\n";
	print "   -d causes debugging output.\n";
	exit(1);
}

