#!/usr/bin/perl

#
#	Verify an Intel 8 bit hex input 
#

#	TODO: VERIFY EACH HEX NUMBER INPUT IS VALID!

use Getopt::Std;

my $debug = 1;
my $self = "IntelCheckHex.pl";
my $eoftype = 1;
my $skipping = 0;
my $first = 1;
my $fileOffset = 0;
my $recordOffset = 0;
my $errorCount = 0;

our($opt_d, $opt_v, $opt_o);

sub usage();
sub dosum($$);

getopts('dvo:') || usage();
$debug = $opt_d;
$verbose = $opt_v;

if($opt_o) {
	$imageFile = $opt_o;
}

if($#ARGV < 0) {
	usage();
	exit(8);
}

open(INFILE,"<$ARGV[0]") || 
	die("Cannot open input file $ARGV[0]\n");
	
binmode(INFILE);

if($opt_o) {
	open(IMAGEFILE,">$imageFile") || 
		die("Cannot open image output file $imageFile\n");
	binmode(IMAGEFILE);
}

print("$self: Processing file $ARGV[0]\n");

$frame = 0;
$doneflag = 0;

$image[0] = -1;

while(1) {
	
	#
	#	Look for ":" for a start of record (this conveniently
	#	ignores whitespace / newlines / CR between records, too.
	#	

	while($frame ne ":") {
		if(read(INFILE,$ch,1) != 1) {
			print("$self: EOF looking for next record.\n");
			$doneflag = 1;
			last;
		}
		$frame = $ch;

		if(!$first && $frame ne ":" && frame ne "\n" &&
			$frame != "\r" && $frame != chr(0xff) && !$skipping) {
			printf("%s: Expected ':' at offset %d, got %02x\n",
				$self,$fileOffset,ord($ch));
			$skipping = 1;
			++$errorCount;
		}

		$recordOffset = $fileOffset;
		++$fileOffset;
	}

	if($doneflag) {
		last;
	}

	#	The flag character isn't included in the checksum

	$chksum = 0;
	$first = 0;

	#
	#	Found start of record, read byte count, address and type
	#

	if(read(INFILE,$s,2) != 2) {
		print("$self: EOF reading byte count");
		close(INFILE);
		exit(2);
	}
	$fileOffset += 2;

	$count = hex($s);
	# print "Length in chars is $s, computed to be $count\n";
	$chksum = dosum($chksum, $s);
	
	if(read(INFILE,$s,4) != 4) {
		print("$self: EOF reading address.\n");
		close(INFILE);
		exit(2);
	}
	$fileOffset += 4;

	$imageAddress = hex($s);
	$chksum = dosum($chksum,$s);

	if(read(INFILE,$s,2) != 2) {
		print("$self: EOF reading record type\n");
		close(INFILE);
		exit(2);
	}
	$fileOffset += 2;

	$recordType = hex($s);
	$chksum = dosum($chksum,$s);

	printf("%s: Offset %5d: Address %04X, type %02X, Length %d\n",
		$self,$recordOffset,$imageAddress,$recordtype,$count);

	#	Read the data bytes

	for($i = 0; $i < $count; ++$i) {
		if(read(INFILE,$s,2) != 2) {
			print("$self: Error/EOF reading data byte in record.\n");
			close(INFILE);
			exit(2);
		}	
		$fileOffset += 2;

		$mydata = hex($s);
		$chksum = dosum($chksum, $s);

		$image[$imageAddress] = $mydata;
		++$imageAddress;
	}	
	
	#	Read the checksum

	if(read(INFILE,$s,2) != 2) {
		print("$self: EOF reading checksum.\n");
		close(INFILE);
		exit(2);
	}
	$fileOffset += 2;

	$chksum = dosum($chksum,$s);

	if($chksum != 0) {
		printf("%s: Checksum mismatch: %02X\n",$self,$chksum);
		++$errorCount;
	}

	$frame = 0;
	$skipping = 0;

	if($recordType == $eoftype) {
		last;
	}
}

close(INFILE);

if($opt_o && $#image > 0) {
	print "$self: saving image file $imageFile\n";
	for($i = 0; $i < $#image; ++$i) {
		print IMAGEFILE chr($image[$i]);
	}
	close(IMAGEFILE);
}

if($errorCount > 0) {
	print("$self: $errorCount ERRORS FOUND.\n");
}

exit(0);

sub dosum($$) {
	my($sum, $added) = @_;
	my $i;
	
	for($i = 0; $i < length($added); $i += 2) {
		$sum = ($sum + hex(substr($added,$i,2)) ) % 256;
	}
	return($sum);
}

sub usage() {
	print("Usage:  perl $self [-d] [-o imagefile] inputFile ");
}
