#
#	Program to check a PDP 8 .BIN file checksum
#	
#	Derived from C program CKBIN.C
#
#	TODO: recognize address and build image contents.
#

use Getopt::Std;

my $debug = 1;
my $self = "PDP8ChkBin.pl";
my $c, $c1, $c2, $ck, $sum, $state, $ch;
my $ccnt = -1;

our($opt_d, $opt_o);;

sub usage();
sub dosum($$);

getopts('dvo:n:') || usage();
$debug = $opt_d;
$verbose = $opt_v;

if($opt_n) {
	$filecount = $opt_n;
}
else {
	$filecount = 1;
}

if($opt_o) {
	$imageFile = $opt_o;
}

if($#ARGV < 0) {
	usage();
	exit(8);
}

open(INFILE,"<$ARGV[0]") || 
	die("Cannot open input file $ARGV[0]\n");
	
binmode(INFILE);

if($opt_o) {
	open(IMAGEFILE,">$imageFile") || 
		die("Cannot open output file $imageFile\n");
	binmode(IMAGEFILE);
}

print("$self: Processing file $ARGV[0]\n");


for($n = 0; $n < $filecount; ++$n) {

$image[0] = -1;
$state = -1;
$ck = 0;
$ccnt = 0;

while(read(INFILE,$ch,1) == 1) {
	$c = ord($ch);
	++$ccnt;

	if($state == -1) {
		if($c == 0) {
			next;
		}
		else {
			$state = 0;
		}
	}

	if($state == 0) {
		if($c != 0x80) {
			if(($c & 0xc0) == 0xc0) {
				$c1 = $c2 = 0;
				$state = 1;
			}
			else {			
				$c1 = $c;
				$state = 2;
			}
		}
	}

	elsif($state == 1) {
		if($c == 0x80) {
			printf "ck = %o, sum = %o\n",$ck & 0xfff,($c1 << 6) | $c2;
			printf "ccnt = %d (0x%x)\n",$ccnt,$ccnt;
			$state = -1;
			last;
		}
		if(($c & 0xc0) == 0xc0) {
			;
		}
		else {
			$ck += $c1+$c2;
			$c1 = $c;
			$state = 2;
		}
	}

	elsif($state == 2) {
		if($c & 0x80) {
			print STDERR "$self: Trailer or field found in middle of word.\n";
			printf "ccnt = %d (0x%x)\n",$ccnt,$ccnt;
			last;
		}
		else {
			$c2 = $c;
			$state = 1;
		}
	}
	else {
		printf STDERR "ckbin: Illegal state: %d\n",$state;
		last;
	}

}

if($state != -1) {
	print STDERR "$self: EOF before trailer.\n";
}

}	# End of for loop - not indented

close(INFILE);

if($opt_o && $#image > 0) {
	print "$self: saving image file $imageFile\n";
	for($i = 0; $i < $#image; ++$i) {
		print IMAGEFILE chr($image[$i]);
	}
	close(IMAGEFILE);
}


exit(0);


sub usage() {
	print "Usage:  perl $self [-d] [-o imagefile] inputFile \n";
}
